import 'dart:convert';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';
import 'package:flutter/material.dart';
import 'package:sample/main.dart';

import 'home_page.dart';

class PushNotificationsManager {
  PushNotificationsManager._();

  factory PushNotificationsManager() => instance;

  static final PushNotificationsManager instance = PushNotificationsManager._();

  final FirebaseMessaging _firebaseMessaging = FirebaseMessaging.instance;

  String? token;
  bool _initialized = false;

  Future<void> init() async {
    if (!_initialized) {
      //Get iOS request permissions
      await Firebase.initializeApp();
      _firebaseMessaging.requestPermission();
      FirebaseMessaging.onMessage.listen((message) {
        processNotificationAtForeground(message.notification?.title,
            message.notification?.body, message.data);
      });

      FirebaseMessaging.onMessageOpenedApp.listen((RemoteMessage message) {
        processNotificationAtBackground(message.data);
      });

      getToken();
      _initialized = true;
    }
  }

  void processNotificationAtBackground(Map<String, dynamic>? message) {
    print('1) data is $message');
    // homeKey.currentState?.goToUrl( message?["click_url"]);
    navigatorKey.currentState?.pushReplacement(
      MaterialPageRoute(
        builder: (ctx) => HomeScreen(
          url: message?["click_url"],
        ),
      ),
    );
  }

  void processNotificationAtForeground(
      String? heading, String? body, Map<String, dynamic>? message) {
    print('2) data is $message');
    // Map<String, dynamic> data;
    // String title = heading;
    // String msg = body;
  }

  Future<void> getToken() async {
    await _firebaseMessaging.getToken().then((value) {
      token = value;
    });
  }
}
