import 'dart:convert';
import 'dart:developer';
import 'package:flutter/material.dart';
import 'package:flutter_webview_plugin/flutter_webview_plugin.dart';
import 'package:sample/push_notification_manager.dart';
import 'package:http/http.dart' as http;
// ignore: library_prefixes
import 'package:encrypt/encrypt.dart' as EncryptPack;
// ignore: library_prefixes
import 'package:crypto/crypto.dart' as CryptoPack;

class ViewPage extends StatefulWidget {
  const ViewPage({Key? key}) : super(key: key);

  @override
  State<ViewPage> createState() => _ViewPageState();
}

class _ViewPageState extends State<ViewPage> {
  final ValueNotifier<bool> _loading = ValueNotifier<bool>(false);
  final flutterWebViewPlugin = FlutterWebviewPlugin();
  final Set<JavascriptChannel> jsChannels = {
    JavascriptChannel(
        name: 'userid',
        onMessageReceived: (JavascriptMessage message) {
          print('message received ${message.message}');
        }),
  };


  @override
  void initState() {
    flutterWebViewPlugin.onStateChanged.listen((event) {
      print('event type is ${event.type}');
      if (event.type == WebViewState.shouldStart) {
        _loading.value = true;
      } else if (event.type == WebViewState.finishLoad) {
        _loading.value = false;
      }
    });
    flutterWebViewPlugin.onProgressChanged.listen((event) {
      if (event == 1.0) {
        _loading.value = false;
      }
    });
    // TODO: implement initState
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
      onWillPop: () async {
        bool canGoBack = await flutterWebViewPlugin.canGoBack();
        if (canGoBack) {
          flutterWebViewPlugin.goBack();
        }
        return !canGoBack;
      },
      child: SafeArea(
        child: Stack(
          alignment: Alignment.center,
          children: [
            WebviewScaffold(
              url: "https://sample.com/",
              javascriptChannels: {
                JavascriptChannel(
                  name: "messageHandler",
                  onMessageReceived: (JavascriptMessage message) async {
                    var msg = message.message;
                    var data = jsonDecode(message.message);

                    var userid = data["userid"];

                    String strPwd =
                        "d766f45c5b4566e170b3e7e7678de0b7"; //secret key generated in step 1 above

                    String token =
                        PushNotificationsManager.instance.token ?? '';

                    final iv = EncryptPack.IV.fromLength(16);

                    final key = EncryptPack.Key.fromUtf8(strPwd); //hardcode

                    final encrypter = EncryptPack.Encrypter(
                        EncryptPack.AES(key, mode: EncryptPack.AESMode.cbc));
                    final encrypted =
                        encrypter.encrypt(token.toString(), iv: iv);
                    var hmacSha256 = CryptoPack.Hmac(
                        CryptoPack.sha256, utf8.encode(strPwd)); // HMAC-SHA256
                    var hmacstring =
                        hmacSha256.convert(utf8.encode(token.toString()));
                    var encryptedsubscription = encrypted.base64 +
                        ":" +
                        iv.base64 +
                        ":" +
                        hmacstring.toString() +
                        ":" +
                        hmacstring.toString();
                    var body = {
                      'token': encryptedsubscription,
                      'userid': userid.toString()
                    };

                    var responseOfAPI = await http.post(
                      Uri.parse(
                          'https://sample.com/wp-json/PNFPBpush/v1/subscriptiontoken'),
                      body: body,
                    );
                    log(responseOfAPI.body);
                    Map<String, dynamic> fResponse =
                        json.decode(responseOfAPI.body);
                  },
                ),
                JavascriptChannel(
                  name: 'unsubscribeGroupid',
                  onMessageReceived: (JavascriptMessage message) async {
                    print('messa ge is ${message.message}');
                    // Here you can take message.message and use
                    // your string from webview
                    /*Fluttertoast.showToast(
                              msg: message.message.toString(),
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 25,
                              fontSize: 16.0
                          );*/

                    String strPwd = "d766f45c5b7976e170b3e7e751ade0b7";

                    var groupid = message.message.toString();

                    final iv = EncryptPack.IV.fromLength(16);

                    final key = EncryptPack.Key.fromUtf8(strPwd); //hardcode

                    final encrypter = EncryptPack.Encrypter(
                        EncryptPack.AES(key, mode: EncryptPack.AESMode.cbc));

                    final encrypted = encrypter.encrypt(
                        PushNotificationsManager.instance.token ?? '',
                        iv: iv);

                    var hmacSha256 = CryptoPack.Hmac(
                        CryptoPack.sha256, utf8.encode(strPwd)); // HMAC-SHA256

                    var hmacstring = hmacSha256.convert(utf8
                        .encode(PushNotificationsManager.instance.token ?? ''));

                    var encryptedsubscription = encrypted.base64 +
                        ":" +
                        iv.base64 +
                        ":" +
                        hmacstring.toString() +
                        ":" +
                        hmacstring.toString();

                    var url = Uri.parse(
                        'https://sample.com/wp-json/PNFPBpush/v1/subscriptiontoken');

                    var response = await http.post(url, body: {
                      'token': encryptedsubscription,
                      'subscription-type': 'unsubscribe-group',
                      'groupid': groupid,
                      'cookievalue': ''
                    });

                    var outputresponse =
                        response.statusCode.toString() + ' ' + response.body;
                    log(response.body);

                    /*Fluttertoast.showToast(
                              msg: outputresponse,
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 35,
                              fontSize: 16.0
                          );*/
                  },
                ),
                JavascriptChannel(
                  name: 'subscribeGroupid',
                  onMessageReceived: (JavascriptMessage message2) async {
                    // Here you can take message.message and use
                    // your string from webview
                    /*Fluttertoast.showToast(
                              msg: message2.message.toString(),
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 15,
                              fontSize: 16.0
                          );*/

                    String strPwd = "d766f45c5b7976e170b3e7e751ade0b7";
                    print('messa ge2 is ${message2.message}');

                    var groupid = message2.message.toString();

                    final iv = EncryptPack.IV.fromLength(16);

                    final key = EncryptPack.Key.fromUtf8(strPwd); //hardcode

                    final encrypter = EncryptPack.Encrypter(
                        EncryptPack.AES(key, mode: EncryptPack.AESMode.cbc));

                    final encrypted = encrypter.encrypt(
                        PushNotificationsManager.instance.token ?? '',
                        iv: iv);

                    var hmacSha256 = CryptoPack.Hmac(
                        CryptoPack.sha256, utf8.encode(strPwd)); // HMAC-SHA256

                    var hmacstring = hmacSha256.convert(utf8
                        .encode(PushNotificationsManager.instance.token ?? ''));

                    var encryptedsubscription = encrypted.base64 +
                        ":" +
                        iv.base64 +
                        ":" +
                        hmacstring.toString() +
                        ":" +
                        hmacstring.toString();

                    var url = Uri.parse(
                        'https://sample.com/wp-json/PNFPBpush/v1/subscriptiontoken');

                    var response = await http.post(url, body: {
                      'token': encryptedsubscription,
                      'subscription-type': 'subscribe-group',
                      'groupid': groupid,
                      'cookievalue': ''
                    });

                    var outputresponse =
                        response.statusCode.toString() + ' ' + response.body;
                    log(response.body);
                    /*Fluttertoast.showToast(
                              msg: outputresponse,
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 35,
                              fontSize: 16.0
                          );*/
                  },
                ),
              },
              initialChild: Container(
                color: Colors.white,
                child: const Center(
                  child: CircularProgressIndicator(),
                ),
              ),
            ),
            Positioned(
              child: ValueListenableBuilder<bool>(
                  valueListenable: _loading,
                  builder: (context, loading, _) {
                    return loading
                        ? const CircularProgressIndicator()
                        : const SizedBox();
                  }),
            )
          ],
        ),
      ),
    );
  }

  pushTokenSubmit(String userid) async {
    print('called');
    String strPwd =
        "d766f45c5b7976e170b3e7e751ade0b7"; //secret key generated in step 1 above

    String token = PushNotificationsManager.instance.token ?? '';
    print('token is $token');

    final iv = EncryptPack.IV.fromLength(16);

    final key = EncryptPack.Key.fromUtf8(strPwd); //hardcode

    final encrypter = EncryptPack.Encrypter(
        EncryptPack.AES(key, mode: EncryptPack.AESMode.cbc));
    final encrypted = encrypter.encrypt(token.toString(), iv: iv);
    var hmacSha256 =
        CryptoPack.Hmac(CryptoPack.sha256, utf8.encode(strPwd)); // HMAC-SHA256
    var hmacstring = hmacSha256.convert(utf8.encode(token.toString()));
    var encryptedsubscription = encrypted.base64 +
        ":" +
        iv.base64 +
        ":" +
        hmacstring.toString() +
        ":" +
        hmacstring.toString();
    var body = {'token': encryptedsubscription, 'userid': userid};
    print('body is $body');
    var responseOfAPI = await http.post(
      Uri.parse(
          'https://happypainting.me/wp-json/PNFPBpush/v1/subscriptiontoken'),
      body: body,
    );
    log(responseOfAPI.body);
    Map<String, dynamic> fResponse = json.decode(responseOfAPI.body);
  }
}
/*
class WebViewContainer extends StatefulWidget {
  final url;
  WebViewContainer(this.url);
  @override
  createState() => _WebViewContainerState(this.url);
}

class _WebViewContainerState extends State<WebViewContainer> {
  var _url;
  final _key = UniqueKey();
  late WebViewController _pnfpbController;
  _WebViewContainerState(this._url);
  @override
  Widget build(BuildContext context) {
    return Scaffold(
        body: Column(
      children: [
        Expanded(
            child: WebView(
          key: _key,
          javascriptMode: JavascriptMode.unrestricted,
          onWebViewCreated: (controller) {
            _pnfpbController = controller;
          },
          initialUrl: _url,
          javascriptChannels: {
            JavascriptChannel(
              name: 'unsubscribeGroupid',
              onMessageReceived: (JavascriptMessage message) async {
                print('messa ge is ${message.message}');
                // Here you can take message.message and use
                // your string from webview
                /*Fluttertoast.showToast(
                              msg: message.message.toString(),
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 25,
                              fontSize: 16.0
                          );*/

                String strPwd = "1c095925a3f5fb27548cd09b2bc37293";

                var groupid = message.message.toString();

                final iv = EncryptPack.IV.fromLength(16);

                final key = EncryptPack.Key.fromUtf8(strPwd); //hardcode

                final encrypter = EncryptPack.Encrypter(
                    EncryptPack.AES(key, mode: EncryptPack.AESMode.cbc));

                final encrypted = encrypter.encrypt(
                    PushNotificationsManager.instance.token ?? '',
                    iv: iv);

                var hmacSha256 = CryptoPack.Hmac(
                    CryptoPack.sha256, utf8.encode(strPwd)); // HMAC-SHA256

                var hmacstring = hmacSha256.convert(
                    utf8.encode(PushNotificationsManager.instance.token ?? ''));

                var encryptedsubscription = encrypted.base64 +
                    ":" +
                    iv.base64 +
                    ":" +
                    hmacstring.toString() +
                    ":" +
                    hmacstring.toString();

                var url = Uri.parse(
                    'https://www.muraliwebworld.com/wp-json/PNFPBpush/v1/subscriptiontoken');

                var response = await http.post(url, body: {
                  'token': encryptedsubscription,
                  'subscription-type': 'unsubscribe-group',
                  'groupid': groupid,
                  'cookievalue': ''
                });

                var outputresponse =
                    response.statusCode.toString() + ' ' + response.body;

                /*Fluttertoast.showToast(
                              msg: outputresponse,
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 35,
                              fontSize: 16.0
                          );*/
              },
            ),
            JavascriptChannel(
              name: 'subscribeGroupid',
              onMessageReceived: (JavascriptMessage message2) async {
                // Here you can take message.message and use
                // your string from webview
                /*Fluttertoast.showToast(
                              msg: message2.message.toString(),
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 15,
                              fontSize: 16.0
                          );*/

                String strPwd = "d766f45c5b7976e170b3e7e751ade0b7";
                print('messa ge2 is ${message2.message}');

                var groupid = message2.message.toString();

                final iv = EncryptPack.IV.fromLength(16);

                final key = EncryptPack.Key.fromUtf8(strPwd); //hardcode

                final encrypter = EncryptPack.Encrypter(
                    EncryptPack.AES(key, mode: EncryptPack.AESMode.cbc));

                final encrypted = encrypter.encrypt(
                    PushNotificationsManager.instance.token ?? '',
                    iv: iv);

                var hmacSha256 = CryptoPack.Hmac(
                    CryptoPack.sha256, utf8.encode(strPwd)); // HMAC-SHA256

                var hmacstring = hmacSha256.convert(
                    utf8.encode(PushNotificationsManager.instance.token ?? ''));

                var encryptedsubscription = encrypted.base64 +
                    ":" +
                    iv.base64 +
                    ":" +
                    hmacstring.toString() +
                    ":" +
                    hmacstring.toString();

                var url = Uri.parse(
                    'https://www.muraliwebworld.com/wp-json/PNFPBpush/v1/subscriptiontoken');

                var response = await http.post(url, body: {
                  'token': encryptedsubscription,
                  'subscription-type': 'subscribe-group',
                  'groupid': groupid,
                  'cookievalue': ''
                });

                var outputresponse =
                    response.statusCode.toString() + ' ' + response.body;

                /*Fluttertoast.showToast(
                              msg: outputresponse,
                              toastLength: Toast.LENGTH_LONG,
                              gravity: ToastGravity.CENTER,
                              timeInSecForIosWeb: 35,
                              fontSize: 16.0
                          );*/
              },
            ),
          },
          onPageFinished: (url) {
            _pnfpbController
                .runJavascriptReturningResult('PNFPB_from_Flutter_mobileapp("' +
                    PushNotificationsManager.instance.token! +
                    '")')
                .then((value) => print('value is $value'));
          },
          // onProgress: (url) {
          //   _pnfpbController
          //       .runJavascriptReturningResult('PNFPB_from_Flutter_mobileapp("' +
          //           PushNotificationsManager.instance.token! +
          //           '")')
          //       .then((value) => print('value is $value'));
          // },
        ))
      ],
    ));
  }
}*/



